import numpy as np
import copy, os
from .GUI_ClassFun import *
from .CPU_Agents import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

class Panel_Output(QWidget):
    def __init__(self, img_raw, img_bin, map, nc, nr, anchors):
        '''
        '''
        super().__init__()
        self.field = Field(img_raw=img_raw, img_bin=img_bin, map=map, nc=nc, nr=nr, anchors=anchors)
        self.layout = QHBoxLayout()
        '''left side'''
        self.pn_left = QWidget()
        self.lo_left = QGridLayout()
        self.wg_img = Widget_Seg(field = self.field)
        self.bt_1 = QPushButton("RGB (J)")
        self.bt_2 = QPushButton("Binary (K)")
        '''right side'''
        self.pn_right = QWidget()
        self.lo_right = QGridLayout()
        self.wg_data = Widget_Data()
        self.lb_project = QLabel()
        self.fd_project = QLineEdit()
        self.lb_output = QLabel()
        self.fd_output = QLineEdit()
        self.bt_output = QPushButton()
        '''ui'''
        self.initUI()
    def initUI(self):
        # button
        self.bt_1.pressed.connect(self.wg_img.switch_raw)
        self.bt_1.released.connect(self.wg_img.switch_seg)
        self.bt_2.pressed.connect(self.wg_img.switch_bin)
        self.bt_2.released.connect(self.wg_img.switch_seg)
        # project
        font = self.fd_project.font()
        font.setPointSize(25)
        fm = QFontMetrics(font)
        self.fd_project.setFixedHeight(fm.height())
        self.fd_output.setFixedHeight(fm.height())
        self.lb_project.setText("Prefix")
        self.fd_project.setText("GRID")
        self.lb_output.setText("Output Path")
        self.fd_output.setText(os.path.expanduser("~"))
        self.bt_output.setText("Browse")
        self.bt_output.clicked.connect(self.assign_PathOut)
        '''layout'''
        # left
        self.lo_left.addWidget(self.wg_img, 0, 0, 1, 2)
        self.lo_left.addWidget(self.bt_1, 1, 0)
        self.lo_left.addWidget(self.bt_2, 1, 1)
        self.pn_left.setLayout(self.lo_left)
        # right
        self.lo_right.addWidget(self.wg_data, 0, 0, 1, 3)
        self.lo_right.addWidget(self.lb_project, 1, 0)
        self.lo_right.addWidget(self.fd_project, 1, 1)
        self.lo_right.addWidget(self.lb_output, 2, 0)
        self.lo_right.addWidget(self.fd_output, 2, 1)
        self.lo_right.addWidget(self.bt_output, 2, 2)
        self.pn_right.setLayout(self.lo_right)
        # policy
        policy_right = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        policy_right.setHorizontalStretch(1)
        self.pn_right.setSizePolicy(policy_right)
        policy_left = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        policy_left.setHorizontalStretch(2)
        self.pn_left.setSizePolicy(policy_left)
        # assemble
        self.layout.addWidget(self.pn_left)
        self.layout.addWidget(self.pn_right)
        self.setLayout(self.layout)
        self.show()
    def assign_PathOut(self):
        path = QFileDialog().getExistingDirectory(self, "", "", QFileDialog.ShowDirsOnly)
        self.fd_output.setText(path)
    def paint_grid(self, qimg):
        """
        """
        pen = QPen()
        pen.setWidth(1)
        pen.setColor(Qt.red)
        painter = QPainter(qimg)
        painter.setPen(pen)
        painter.setBrush(Qt.transparent)
        for row in range(self.field.nrow):
            for col in range(self.field.ncol):
                agent = self.field.get_agent(row, col)
                rect = agent.get_rect()
                painter.drawRect(rect)
        painter.end()
    def out_raw(self, path):
        """
        """
        img = self.wg_img.img_raw[:,:,:3].copy()
        self.wg_img.make_rgb_img(img)
        qimg = self.wg_img.qimg
        self.paint_grid(qimg)
        save_img(qimg, path+"_raw")
        self.wg_img.switch_seg()
    def out_seg(self, path):
        """
        """
        self.wg_img.make_rgb_img(self.wg_img.img_seg)
        qimg = self.wg_img.qimg
        self.paint_grid(qimg)
        save_img(qimg, path+"_seg")
        self.wg_img.switch_seg()
    def out_bin(self, path):
        """
        """
        self.wg_img.make_bin_img(self.wg_img.img_bin)
        qimg = self.wg_img.qimg
        self.paint_grid(qimg)
        save_img(qimg, path+"_kmean")
        self.wg_img.switch_seg()
    def output(self):
        """
        """
        path_out = self.fd_output.text()+"/"+self.fd_project.text()
        # figure
        self.out_raw(path=path_out)
        self.out_seg(path=path_out)
        self.out_bin(path=path_out)
        # dataframe
        df = self.field.get_DF()
        if self.wg_data.item1s[0][0].checkState()==Qt.Checked:
            idx = self.field.get_index(ch_1=3, ch_2=0, isContrast=True, name_index="NDVI")
            df = pd.merge(df, idx, on='var', how='left')
        if self.wg_data.item1s[1][0].checkState()==Qt.Checked:
            idx = self.field.get_index(ch_1=3, ch_2=1, isContrast=True, name_index="GNDVI")
            df = pd.merge(df, idx, on='var', how='left')
        if self.wg_data.item1s[2][0].checkState()==Qt.Checked:
            idx = self.field.get_index(ch_1=1, ch_2=0, isContrast=True, name_index="NDGI")
            df = pd.merge(df, idx, on='var', how='left')
        if self.wg_data.item1s[3][0].checkState()==Qt.Checked:
            idx = self.field.get_index(ch_1=3, ch_2=0, ch_3=1, isThree=True, name_index="CNDVI")
            df = pd.merge(df, idx, on='var', how='left')
        if self.wg_data.item1s[4][0].checkState()==Qt.Checked:
            idx = self.field.get_index(ch_1=3, ch_2=0, isRatio=True, name_index="RVI")
            df = pd.merge(df, idx, on='var', how='left')
        if self.wg_data.item1s[5][0].checkState()==Qt.Checked:
            idx = self.field.get_index(ch_1=3, ch_2=1, isRatio=True, name_index="GRVI")
            df = pd.merge(df, idx, on='var', how='left')
        df.to_csv(path_out+"_data.csv", index=False)


class Widget_Seg(Widget_Img):
    def __init__(self, field):
        '''
        '''
        super().__init__(field.img_raw)
        self.setMouseTracking(True)
        self.zoom = 1
        '''attr'''
        # basic
        self.field = field
        self.agents_reset = copy.deepcopy(self.field.agents)
        self.img_raw = field.img_raw
        self.img_bin = field.img_bin
        # painter
        self.is_fit_width = False
        self.pt_st_img = 0
        self.ratio = 0
        # mouse
        self.agent_click = None
        self.dir = None
        # ui
        self.initUI()
    def initUI(self):
        '''generate seg image'''
        img_temp = self.img_bin.reshape(self.img_bin.shape[0], self.img_bin.shape[1], 1)
        self.img_seg = np.multiply(self.img_raw[:,:,:3], img_temp).copy()
        self.img_seg[(self.img_seg.mean(axis=2)==0), :] = self.img_seg.max()
        self.switch_seg()
        self.show()
    def mousePressEvent(self, event):
        pos = event.pos()
        for row in range(self.field.nrow):
            for col in range(self.field.ncol):
                agent = self.field.get_agent(row, col)
                rect = agent.get_rect()
                if self.is_fit_width:
                    self.ratio = self.width()/self.qimg.width()
                    rec_agent = QRect(rect.x()*self.ratio, rect.y()*self.ratio+self.pt_st_img, rect.width()*self.ratio, rect.height()*self.ratio)
                else:
                    self.ratio = self.height()/self.qimg.height()
                    rec_agent = QRect(rect.x()*self.ratio+self.pt_st_img, rect.y()*self.ratio, rect.width()*self.ratio, rect.height()*self.ratio)
                if rec_agent.contains(pos):
                    bd_W = rec_agent.x()
                    bd_N = rec_agent.y()
                    bd_E = bd_W + rec_agent.width()
                    bd_S = bd_N + rec_agent.height()
                    dis_W = abs(pos.x()-bd_W)
                    dis_N = abs(pos.y()-bd_N)
                    dis_E = abs(pos.x()-bd_E)
                    dis_S = abs(pos.y()-bd_S)
                    # print("W:%.2f, N:%.2f, E:%.2f, S:%.2f" %(dis_W, dis_N, dis_E, dis_S))
                    dir_idx = np.argmin(np.array([dis_N, dis_W, dis_S, dis_E]))
                    if dir_idx==0:
                        self.dir = Dir.NORTH
                    elif dir_idx==1:
                        self.dir = Dir.WEST
                    elif dir_idx==2:
                        self.dir = Dir.SOUTH
                    elif dir_idx==3:
                        self.dir = Dir.EAST
                    self.agent_click = agent
                    break
        # mag module
        if event.button() == Qt.RightButton:
            self.zoom = (self.zoom+1)%3
            self.mouseMoveEvent(event)
    def mouseMoveEvent(self, event):
        pos = event.pos()
        if event.buttons() == Qt.LeftButton:
            # adjust the border
            if self.is_fit_width:
                if self.dir==Dir.NORTH or self.dir==Dir.SOUTH:
                    value = (pos.y()-self.pt_st_img)/self.ratio
                elif self.dir==Dir.WEST or self.dir==Dir.EAST:
                    value = pos.x()/self.ratio
            else:
                if self.dir==Dir.NORTH or self.dir==Dir.SOUTH:
                    value = pos.y()/self.ratio
                elif self.dir==Dir.WEST or self.dir==Dir.EAST:
                    value = (pos.x()-self.pt_st_img)/self.ratio
            self.agent_click.set_border(self.dir, value)
        # mag module
        if self.zoom!=0:
            magnifying_glass(self, pos, area=200, zoom=self.zoom*2)
        else:
            self.setCursor(QCursor(Qt.ArrowCursor))
        self.repaint()
    def paintEvent(self, paint_event):
        painter = QPainter(self)
        super().paintImage(painter)
        pen = QPen()
        pen.setWidth(1)
        pen.setColor(Qt.red)
        painter.setPen(pen)
        painter.setBrush(Qt.transparent)
        for row in range(self.field.nrow):
            for col in range(self.field.ncol):
                agent = self.field.get_agent(row, col)
                rect = agent.get_rect()
                pt_x, pt_y = agent.get_coordinate()
                if self.is_fit_width:
                    self.ratio = self.width()/self.qimg.width()
                    rec_agent = QRect(rect.x()*self.ratio, rect.y()*self.ratio+self.pt_st_img, rect.width()*self.ratio, rect.height()*self.ratio)
                    draw_cross(pt_x*self.ratio, pt_y*self.ratio+self.pt_st_img, painter)
                else:
                    self.ratio = self.height()/self.qimg.height()
                    rec_agent = QRect(rect.x()*self.ratio+self.pt_st_img, rect.y()*self.ratio, rect.width()*self.ratio, rect.height()*self.ratio)
                    draw_cross(pt_x*self.ratio+self.pt_st_img, pt_y*self.ratio, painter)
                painter.drawRect(rec_agent)
        painter.end()
    def switch_raw(self):
        img = self.img_raw[:,:,:3].copy()
        super().make_rgb_img(img)
        self.repaint()
    def switch_bin(self):
        super().make_bin_img(self.img_bin)
        self.repaint()
    def switch_seg(self):
        super().make_rgb_img(self.img_seg)
        self.repaint()
    def reset(self):
        self.field.agents = None
        self.field.agents = copy.deepcopy(self.agents_reset)
        self.repaint()


class Widget_Data(QWidget):
    def __init__(self):
        super().__init__()
        self.view = QTreeView(self)
        self.model = QStandardItemModel()
        self.initUI()
    def initUI(self):
        self.model.setHorizontalHeaderLabels(['Output', 'Data'])
        self.rootItem = self.model.invisibleRootItem()
        self.buildTree()
        self.view.setModel(self.model)
        layout = QHBoxLayout(self)
        layout.addWidget(self.view)
        self.setLayout(layout)
        self.view.expandAll()
        self.show()
    def buildTree(self):
        item0 = [QStandardItem('Figures'), QStandardItem(' ')]
        self.item0s = []
        self.item0s.append([QStandardItem('Raw'), QStandardItem('Raw image')])
        self.item0s.append([QStandardItem('Segmentation'), QStandardItem('Extraction of area of interest (AOI)')])
        self.item0s.append([QStandardItem('K-mean'), QStandardItem('Visualization of K-mean results')])
        item1 = [QStandardItem('Data Frame'), QStandardItem(' ')]
        self.item1s = []
        self.item1s.append([QStandardItem('NDVI'), QStandardItem('(NIR-Red) / (NIR+Red)')])
        self.item1s.append([QStandardItem('GNDVI'), QStandardItem('(NIR-Green) / (NIR+Green)')])
        self.item1s.append([QStandardItem('NDGI'), QStandardItem('(Green-Red) / (Green+Red)')])
        self.item1s.append([QStandardItem('CNDVI'), QStandardItem('(2NIR-Red-Green) / (NIR+Red+Green)')])
        self.item1s.append([QStandardItem('RVI'), QStandardItem('NIR / Red')])
        self.item1s.append([QStandardItem('GRVI'), QStandardItem('NIR / Green')])
        for i in range(3):
            item0[0].appendRow(self.item0s[i])
            self.item0s[i][0].setCheckable(True)
            self.item0s[i][0].setCheckState(Qt.Checked)
        for i in range(6):
            item1[0].appendRow(self.item1s[i])
            # if (i==0) or (i==2):
            self.item1s[i][0].setCheckState(Qt.Checked)
            self.item1s[i][0].setCheckable(True)
        self.rootItem.appendRow(item0)
        self.rootItem.appendRow(item1)



import numpy as np
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

class Widget_Img(QWidget):
    '''
    Will keep imgRaw, imgVis and imgQmap
    '''
    def __init__(self, img):
        super().__init__()
        '''attr'''
        self.img_raw = img
        self.img_vis = img[:,:,:3].copy()
    def make_rgb_img(self, img):
        h, w = img.shape[0], img.shape[1]
        qImg = QImage(img.astype(np.uint8), w, h, w*3, QImage.Format_RGB888)
        self.qimg = QPixmap(qImg)
    def make_bin_img(self, img):
        h, w = img.shape[0], img.shape[1]
        qImg = QImage(img.astype(np.uint8), w, h, w*1, QImage.Format_Indexed8)
        qImg.setColor(0, qRgb(60, 5, 74))
        qImg.setColor(1, qRgb(241, 225, 29))
        self.qimg = QPixmap(qImg)
    def make_idx8_img(self, img, k):
        colormap = [qRgb(60, 5, 74),
                    qRgb(241, 225, 29),
                    qRgb(52, 177, 106),
                    qRgb(46, 85, 130),
                    qRgb(224, 79, 59),
                    qRgb(247, 248, 177)]
        h, w = img.shape[0], img.shape[1]
        qImg = QImage(img.astype(np.uint8), w, h, w*1, QImage.Format_Indexed8)
        for i in range(k):
            qImg.setColor(i, colormap[i])
        self.qimg = QPixmap(qImg)
    def paintImage(self, painter):
        painter.setRenderHint(QPainter.Antialiasing, True)
        self.size_img = self.qimg.size().scaled(self.rect().size(), Qt.KeepAspectRatio)
        if self.size_img.width()==self.width():
            self.is_fit_width = True
            self.pt_st_img = int((self.height()-self.size_img.height())/2)
            painter.drawPixmap(0, self.pt_st_img, self.size_img.width(), self.size_img.height(), self.qimg)
        elif self.size_img.height()==self.height():
            self.is_fit_width = False
            self.pt_st_img = int((self.width()-self.size_img.width())/2)
            painter.drawPixmap(self.pt_st_img, 0, self.size_img.width(), self.size_img.height(), self.qimg)

def magnifying_glass(widget, pos, area=200, zoom=4):
    size = int(area/zoom)
    pixmap = widget.grab(QRect(QPoint(pos.x()-int(size/2), pos.y()-int(size/2)), QSize(size, size)))
    try:
        rate_screen = size/pixmap.width()
        # print(pixmap.rect())
        # print(pixmap.size())
        # print(pixmap.rect().center()*rate_screen)
        pixmap = pixmap.scaled(int(area/rate_screen), int(area/rate_screen))
        # pixmap = pixmap_temp
        painter = QPainter(pixmap)
        '''Rect'''
        pen = QPen()
        pen.setWidth(2)
        pen.setColor(Qt.black)
        painter.setPen(pen)
        # define rect
        rect = QRect(QPoint(0,0), pixmap.size()*rate_screen)
        # draw rect
        painter.drawRect(rect)
        '''Cursor'''
        pen.setWidth(3)
        pen.setColor(Qt.red)
        painter.setPen(pen)
        size_m = 10
        # define lines
        line1 = QLine(QPoint(size_m, 0), QPoint(-size_m, 0))
        line2 = QLine(QPoint(0, size_m), QPoint(0, -size_m))
        line1.translate(pixmap.rect().center()*rate_screen-QPoint(0,0))
        line2.translate(pixmap.rect().center()*rate_screen-QPoint(0,0))
        # draw lines
        painter.drawLine(line1)
        painter.drawLine(line2)
        '''finish'''
        painter.end()
        cursor = QCursor(pixmap)
        widget.setCursor(cursor)
    except:
        '''not in a valid region'''

def save_img(qimg, path):
    w, h = qimg.width(), qimg.height()
    file = QFile(path+".jpg");
    file.open(QIODevice.WriteOnly);
    qimg.save(file, "JPG");


def draw_cross(x, y, painter, size_mark=2):
    l1_st_x, l1_st_y = x-size_mark, y-size_mark
    l1_ed_x, l1_ed_y = x+size_mark, y+size_mark
    l2_st_x, l2_st_y = x-size_mark, y+size_mark
    l2_ed_x, l2_ed_y = x+size_mark, y-size_mark
    painter.drawLine(l1_st_x, l1_st_y, l1_ed_x, l1_ed_y)
    painter.drawLine(l2_st_x, l2_st_y, l2_ed_x, l2_ed_y)

def draw_triangle(x, y, dir, painter, range=7, peak=30):
    path = QPainterPath()
    path.moveTo(x, y)
    if dir=='North':
        path.lineTo(x-range, y+peak)
        path.lineTo(x+range, y+peak)
    elif dir=='South':
        path.lineTo(x-range, y-peak)
        path.lineTo(x+range, y-peak)
    elif dir=='West':
        path.lineTo(x+peak, y-range)
        path.lineTo(x+peak, y+range)
    elif dir=='East':
        path.lineTo(x-peak, y-range)
        path.lineTo(x-peak, y+range)
    path.lineTo(x, y)
    painter.drawPath(path)



import numpy as np
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from .GUI_ClassFun import *

class Panel_Cropper(QGroupBox):
    def __init__(self, np_img):
        super().__init__("Image Correction: Please click on the image to assign four anchors")
        self.setStyleSheet("""
        QGroupBox::title{
            subcontrol-origin: margin;
            subcontrol-position: top center;
        }
        """)
        '''attr'''
        self.layout = QVBoxLayout()
        self.wg_img = Widget_ViewCrop(np_img)
        self.initUI()
    def initUI(self):
        self.layout.addWidget(self.wg_img)
        # self.layout.setAlignment(Qt.AlignCenter)
        self.setLayout(self.layout)
        self.show()
    def get_transformed_img(self):
        return self.wg_img.get_transformed_img()

class Widget_ViewCrop(Widget_Img):
    def __init__(self, img):
        super().__init__(img)
        self.setMouseTracking(True)
        self.zoom = 1
        self.ratio = 1
        self.marks = []
        self.n_marks = 0
        self.is_fit_width = True
        self.pt_st_img = 0
        self.size_img = 0
        self.imgH, self.imgW = img.shape[0], img.shape[1]
        self.initUI()
    def initUI(self):
        super().make_rgb_img(self.img_vis)
        self.show()
    def paintEvent(self, paint_event):
        painter = QPainter(self)
        super().paintImage(painter)
        pen = QPen()
        pen.setWidth(3)
        pen.setColor(Qt.red)
        painter.setPen(pen)
        painter.setBrush(Qt.white)
        for pos in self.marks:
            draw_cross(pos.x(), pos.y(), painter, 5)
    def mouseMoveEvent(self, event):
        pos = event.pos()
        if self.zoom!=0:
            magnifying_glass(self, pos, area=200, zoom=self.zoom*2)
        else:
            self.setCursor(QCursor(Qt.ArrowCursor))
    def mousePressEvent(self, event):
        if event.button() == Qt.RightButton:
            self.zoom = (self.zoom+1)%3
            self.mouseMoveEvent(event)
    def mouseReleaseEvent(self, event):
        pt_mouse = event.pos()
        # determine the image boundary
        bd_neg = self.pt_st_img
        if event.button() == Qt.LeftButton:
            if self.is_fit_width:
                pt_1d = pt_mouse.y()
                bd_pos = self.pt_st_img + self.size_img.height()
            else:
                pt_1d = pt_mouse.x()
                bd_pos = self.pt_st_img + self.size_img.width()
            # if the click is in the image
            if (pt_1d >= bd_neg) & (pt_1d <= bd_pos):
                if self.n_marks<4:
                    self.marks.append(pt_mouse)
                    self.n_marks += 1
                else:
                    self.marks = []
                    self.n_marks = 0
            # print('(x:%d, y:%d)' % (pt_mouse.x()*(self.ratio), pt_mouse.y()*(self.ratio)))
            # print("ratio: %.2f" % (self.ratio))
            self.update()
        self.mouseMoveEvent(event)
    def get_transformed_img(self):
        if self.is_fit_width:
            self.ratio = (self.imgW)/(self.width())
            pts = [[pt.x()*(self.ratio), (pt.y()-self.pt_st_img)*(self.ratio)] for pt in self.marks]
        else:
            self.ratio = (self.imgH)/(self.height())
            pts = [[(pt.x()-self.pt_st_img)*(self.ratio), pt.y()*(self.ratio)] for pt in self.marks]
        if len(pts)<4:
            pts = [[0, 0], [0, self.imgH], [self.imgW, 0], [self.imgW, self.imgH]]
        return transform_by_cv2(self.img_raw, pts).copy()


def rotate(pts, angle):
    import math
    ptx = np.array([pts[i, 0] for i in range(len(pts))])
    pty = np.array([pts[i, 1] for i in range(len(pts))])
    qx = math.cos(math.radians(angle))*(ptx)-math.sin(math.radians(angle))*(pty)
    qy = math.sin(math.radians(angle))*(ptx)+math.cos(math.radians(angle))*(pty)
    qpts = [[qx[i], qy[i]] for i in range(len(pts))]
    return np.array(qpts)

def transform_by_cv2(img, pts):
    import cv2
    # convert to opencv cpmatitable
    pts = np.array(pts).astype(np.float32)
    # find four corners
    token = True
    while token:
        try:
            order_x = np.argsort([pts[i, 0] for i in range(4)])
            order_y = np.argsort([pts[i, 1] for i in range(4)])
            pt_NW = pts[order_x[:2][np.isin(order_x[:2], order_y[:2])][0]]
            pt_SW = pts[order_x[:2][np.isin(order_x[:2], order_y[2:])][0]]
            pt_NE = pts[order_x[2:][np.isin(order_x[2:], order_y[:2])][0]]
            pt_SE = pts[order_x[2:][np.isin(order_x[2:], order_y[2:])][0]]
            token = False
        except:
            pts = rotate(pts, 15)
    # generate sorted source point
    pts = np.array([pt_NW, pt_NE, pt_SW, pt_SE])
    # estimate output dimension
    img_W = (sum((pt_NE-pt_NW)**2)**(1/2)+sum((pt_SE-pt_SW)**2)**(1/2))/2
    img_H = (sum((pt_SE-pt_NE)**2)**(1/2)+sum((pt_SW-pt_NW)**2)**(1/2))/2
    while (img_W > 1500):
        img_W /= 2
        img_H /= 2
    shape = (int(img_W), int(img_H))
    # generate target point
    pts2 = np.float32([[0,0],[shape[0], 0],[0, shape[1]],[shape[0],shape[1]]])
    # transformation
    M = cv2.getPerspectiveTransform(pts, pts2)
    dst = cv2.warpPerspective(img, M, (shape[0], shape[1]))
    return dst



import numpy as np
from .GUI_ClassFun import *
from .CPU_ClassFun import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

class Panel_Anchor(QWidget):
    def __init__(self, img, map):
        super().__init__()
        self.setMouseTracking(True)
        self.zoom = 1
        '''attr'''
        # mouse
        self.idx_click = -1
        self.pos_temp = QPoint(0, 0)
        self.state_hand = False
        self.cursor = QCursor(Qt.ArrowCursor)
        # size/rec
        self.size_self = QSize(0, 0)
        self.size_imgPan = QSize(0, 0)
        self.size_img = QSize(0, 0)
        # anchor
        self.acr_c, self.acr_c_raw = 0, 0
        self.acr_r, self.acr_r_raw = 0, 0
        self.rec_acr_c = QRect(0, 0, 0, 0)
        self.rec_acr_r = QRect(0, 0, 0, 0)
        self.x_acr_c, self.y_acr_c = 0, 0
        self.x_acr_r, self.y_acr_r = 0, 0
        # image
        self.img = img
        self.imgH = self.img.shape[0]
        self.imgW = self.img.shape[1]
        self.margin = 70
        self.space = 5
        # button
        self.bt_reset = QPushButton("Reset")
        '''get peaks'''
        acr_r_temp, _ = get_peak(img=self.img, map=map, axis=0)
        acr_c_temp, _ = get_peak(img=self.img, map=map, axis=1)
        self.acr_r = scaleTo0and1(acr_r_temp, self.imgH)
        self.acr_c = scaleTo0and1(acr_c_temp, self.imgW)
        self.acr_r_raw = self.acr_r.copy()
        self.acr_c_raw = self.acr_c.copy()
        '''UI'''
        self.initUI()
    def initUI(self):
        '''mouse'''
        '''image'''
        qImg = QImage(self.img, self.imgW, self.imgH, self.imgW*1, QImage.Format_Indexed8)
        qImg.setColor(0, qRgb(60, 5, 74))
        qImg.setColor(1, qRgb(241, 225, 29))
        self.qimg = QPixmap(qImg)
        '''button'''
        self.bt_reset.clicked.connect(self.reset_Anchors)
        vbox = QVBoxLayout()
        vbox.addStretch(1)
        vbox.addWidget(self.bt_reset)
        box_bt = QHBoxLayout()
        box_bt.addStretch(1)
        box_bt.addLayout(vbox)
        self.setLayout(box_bt)
        self.show()
    def mousePressEvent(self, event):
        pos = event.pos()
        self.pos_temp = QPoint(pos.x(), pos.y())
        if self.rec_acr_c.contains(pos):
            self.idx_click = (np.abs(self.x_acr_c-pos.x())).argmin()
            if event.button() == Qt.RightButton:
                self.acr_c = np.delete(self.acr_c, self.idx_click)
        elif self.rec_acr_r.contains(pos):
            self.idx_click = (np.abs(self.y_acr_r-pos.y())).argmin()
            if event.button() == Qt.RightButton:
                self.acr_r = np.delete(self.acr_r, self.idx_click)
        elif event.button() == Qt.RightButton:
            # mag module
            self.zoom = (self.zoom+1)%3
            self.mouseMoveEvent(event)
    def mouseReleaseEvent(self, event):
        pos = event.pos()
        # add new anchors
        if (pos.x()==self.pos_temp.x())&(pos.y()==self.pos_temp.y())&(event.button()==Qt.LeftButton):
            if self.rec_acr_c.contains(pos):
                correct = 0 if self.is_fit_width else self.pt_st_img
                new_acr_c = (pos.x()-correct)/(self.size_img.width())
                self.acr_c = np.append(self.acr_c, new_acr_c)
            elif self.rec_acr_r.contains(pos):
                correct = self.pt_st_img if self.is_fit_width else 0
                new_acr_r = (pos.y()-correct)/(self.size_img.height())
                self.acr_r = np.append(self.acr_r, new_acr_r)
        self.repaint()
    def mouseMoveEvent(self, event):
        pos = event.pos()
        correctX = 0 if self.is_fit_width else self.pt_st_img
        correctY = self.pt_st_img if self.is_fit_width else 0
        if event.buttons() == Qt.LeftButton:
            if self.rec_acr_c.contains(pos):
                self.acr_c[self.idx_click] = (pos.x()-correctX)/self.size_img.width()
            elif self.rec_acr_r.contains(pos):
                self.acr_r[self.idx_click] = (pos.y()-correctY)/self.size_img.height()
            self.repaint()
        if (self.rec_acr_c.contains(pos)) or (self.rec_acr_r.contains(pos)):
            self.cursor = QCursor(Qt.PointingHandCursor)
            self.setCursor(self.cursor)
        else:
            # mag module
            if self.zoom!=0:
                magnifying_glass(self, pos, area=200, zoom=self.zoom*2)
            else:
                self.setCursor(QCursor(Qt.ArrowCursor))
    def reset_Anchors(self):
        self.acr_c = self.acr_c_raw
        self.acr_r = self.acr_r_raw
        self.repaint()
    def paintEvent(self, paint_event):
        '''painter'''
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing, True)
        pen = QPen()
        pen.setWidth(3)
        pen.setColor(Qt.red)
        painter.setPen(pen)
        painter.setBrush(Qt.red)
        '''sort array'''
        self.acr_c = np.sort(self.acr_c)
        self.acr_r = np.sort(self.acr_r)
        '''size info'''
        self.size_self = self.rect().size()
        self.size_imgPan = QSize(self.size_self.width()-self.margin, self.size_self.height()-self.margin)
        self.size_img = self.qimg.size().scaled(self.size_imgPan, Qt.KeepAspectRatio)
        '''Check the image side'''
        if self.size_img.width()==self.size_imgPan.width():
            '''image'''
            self.is_fit_width = True
            self.pt_st_img = int((self.size_imgPan.height()-self.size_img.height())/2)
            painter.drawPixmap(0, self.pt_st_img, self.size_img.width(), self.size_img.height(), self.qimg)
            '''anchor X'''
            self.x_acr_c = (self.acr_c*self.size_img.width()).astype(np.int)
            self.y_acr_c = self.pt_st_img+self.size_self.height()+(self.margin/5)
            '''anchor Y'''
            self.x_acr_r = self.size_self.width()-(self.margin*4/5)
            self.y_acr_r = (self.acr_r*self.size_img.height()+self.pt_st_img).astype(np.int)
            '''rect'''
            self.rec_acr_c = QRect(0, self.pt_st_img+self.size_img.height()+self.space, self.size_img.width(), self.margin)
            self.rec_acr_r = QRect(self.size_img.width()+self.space, self.pt_st_img, self.margin, self.size_img.height())
        elif self.size_img.height()==self.size_imgPan.height():
            '''image'''
            self.is_fit_width = False
            self.pt_st_img = int((self.size_imgPan.width()-self.size_img.width())/2)
            painter.drawPixmap(self.pt_st_img, 0, self.size_img.width(), self.size_img.height(), self.qimg)
            '''anchor X'''
            self.x_acr_c = (self.acr_c*self.size_img.width()+self.pt_st_img).astype(np.int)
            self.y_acr_c = self.size_self.height()-(self.margin*4/5)
            '''anchor Y'''
            self.x_acr_r = self.pt_st_img+self.size_img.width()+(self.margin/5)
            self.y_acr_r = (self.acr_r*self.size_img.height()).astype(np.int)
            '''rect'''
            self.rec_acr_c = QRect(self.pt_st_img, self.size_img.height()+self.space, self.size_img.width(), self.margin-5)
            self.rec_acr_r = QRect(self.pt_st_img+self.size_img.width()+self.space, 0, self.margin-5, self.size_img.height())
        '''anchor'''
        # side
        for posX in self.x_acr_c:
            draw_triangle(posX, self.y_acr_c+self.space, "North", painter)
        for posY in self.y_acr_r:
            draw_triangle(self.x_acr_r+self.space, posY, "West", painter)
        # image
        pen.setWidth(3)
        pen.setColor(Qt.red)
        painter.setPen(pen)
        painter.setBrush(Qt.white)
        for posX in self.x_acr_c:
            for posY in self.y_acr_r:
                draw_cross(posX, posY, painter)
        '''rect'''
        pen.setWidth(1)
        pen.setColor(Qt.black)
        painter.setPen(pen)
        painter.setBrush(Qt.transparent)
        painter.drawRect(self.rec_acr_c)
        painter.drawRect(self.rec_acr_r)
    def get_anchors(self):
        imgW = self.qimg.size().width()
        imgH = self.qimg.size().height()
        anchor_x_org = scaleToOrig(self.acr_c, imgW)
        anchor_y_org = scaleToOrig(self.acr_r, imgH)
        list = []
        for posY in anchor_y_org:
            for posX in anchor_x_org:
                list.append(dict(x=posX, y=posY))
        return list, len(self.acr_c), len(self.acr_r)


def scaleTo0and1(array, length):
    array = np.array(array)
    return (array-0)/(length-0)

def scaleToOrig(array, length):
    array = np.array(array)
    return array*(length-0)+0
